/**
 *#########################################################################
 *
 * A component of the Gatherer application, part of the Greenstone digital
 * library suite from the New Zealand Digital Library Project at the
 * University of Waikato, New Zealand.
 *
 * <BR><BR>
 *
 * Author: John Thompson, Greenstone Digital Library, University of Waikato
 *
 * <BR><BR>
 *
 * Copyright (C) 1999 New Zealand Digital Library Project
 *
 * <BR><BR>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * <BR><BR>
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * <BR><BR>
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *########################################################################
 */

 




package org.greenstone.gatherer.cdm;
/**************************************************************************************
 * Title:        Gatherer
 * Description:  The Gatherer: a tool for gathering and enriching a digital collection.
 * Copyright:    Copyright (c) 2001
 * Company:      The University of Waikato
 * Written:      03/05/02
 * Revised:      17/11/02 - Commenting
 **************************************************************************************/
import org.greenstone.gatherer.msm.MSMUtils;
import org.w3c.dom.Element;
/** This class encapsulates one subcollection entry in the collection configuration file.
* @author John Thompson, Greenstone Digital Library, University of Waikato
* @version 2.3
*/
public class Subcollection
	 implements Comparable {
	 /** A <i>boolean</i> which is <i>true</i> if the condition is an include one, <i>false</i> otherwise. */
	 private boolean include = true;
	 /** Either the fully qualified name of the metadata whose value should be matched against the given expression, or <i>null</i> if you wish to match against the file name. */
	 private String element = null;
	 /** A String containing a Perl expression which is used as the filter for this subcollection. */
	 private String exp = null;
	 /** A series of flags to be used when matching the expression. */
	 private String flags = null;
	 /** A String which is a unique identifier of a subcollection. */
	 private String name = null;
	 /** Constructor.
	  * @param name A <strong>String</strong> which is a unique identifier of a subcollection.
	  * @param pattern A <strong>String</strong> containing the inclusion, the element to filter, a Perl expression which is used as the filter for this subcollection, and a series of argument flags.
	  */
	 public Subcollection(String name, String pattern) {
		  this.name = name;
		  // Have to do some work on pattern.
		  // Remove quote marks.
		  if(pattern.startsWith("\"") || pattern.startsWith("'")) {
				pattern = pattern.substring(1, pattern.length() - 1);
		  }
		  // Test for exclusion
		  if(pattern.startsWith("!")) {
				this.include = false;
				pattern = pattern.substring(1, pattern.length());
		  }
		  else {
				this.include = true;
		  }
		  if(pattern.indexOf("/") != -1) {
				// Now element (which may be Filename)
				this.element = pattern.substring(0, pattern.indexOf("/"));
				if(this.element.toLowerCase().equals("filename")) {
					 this.element = null;
				}
				// Extract Perl expression
				if(pattern.indexOf("/") != pattern.lastIndexOf("/")) {
					 this.exp = pattern.substring(pattern.indexOf("/") + 1,
															pattern.lastIndexOf("/"));
					 this.flags = pattern.substring(pattern.lastIndexOf("/") + 1);
				}
				else {
					 this.exp = pattern.substring(pattern.indexOf("/") + 1);
				}
		  }
		  else {
				// Hmmm. Well I don't know what this is but it isn't a proper subcollection definition. Can't really 'return null' or something. So I'll just not set include, element, exp and flag expressions.
		  }
	 }
	 /** Constructor.
	  * @param name A <strong>String</strong> which is a unique identifier of a subcollection.
	  * @param include A <i>boolean</i> which is <i>true</i> if the condition is an include one, <i>false</i> otherwise.
	  * @param exp A <strong>String</strong> containing a Perl expression which is used as the filter for this subcollection.
	  */
	 public Subcollection(String name, boolean include, String exp, String flags) {
		  this.element = null;
		  this.exp = exp;
		  this.flags = flags;
		  this.include = include;
		  this.name = name;
	 }
	 /** Constructor.
	  * @param name A <strong>String</strong> which is a unique identifier of a subcollection.
	  * @param include A <i>boolean</i> which is <i>true</i> if the condition is an include one, <i>false</i> otherwise.
	  * @param element Either the fully qualified name of the metadata whose value should be matched against the given expression, or <i>null</i> if you wish to match against the file name.
	  * @param exp A <strong>String</strong> containing a Perl expression which is used as the filter for this subcollection.
	  * @param flags A <strong>String</strong> of flags to be taken into account when matching the expression.
	  */
	 public Subcollection(String name, boolean include, String element, String exp, String flags) {
		  this.element = element;
		  this.exp = exp;
		  this.flags = flags;
		  this.include = include;
		  this.name = name;
	 }
	 /** Method to compare two subcollections.
	  * @param object The other subcollection to compare to, as an <strong>Object</strong>.
	  * @return An <i>int</i> which is greater than, equal to or less than zero if the this object is before, equal to, or after the target object respectively.
	  */
	 public int compareTo(Object object) {
		  Subcollection sub = (Subcollection) object;
		  return getName().compareTo(sub.getName());
	 }
	 /** Method to check two subcollections for equality.
	  * @param object The other subcollection to compare to, as an <strong>Object</strong>.
	  * @return A <i>boolean</i> which is <i>true</i> if the subcollections are equal, <i>false</i> otherwise.
	  */
	 public boolean equals(Object object) {
		  if(compareTo(object) == 0) {
				return true;
		  }
		  return false;
	 }
	 /** Method to get the value of element.
	  * @return The value of element as an <strong>String</strong>.
	  */
	 public String getElement() {
		  return element;
	 }
	 /** Method to get the value of expression.
	  * @return The value of exp as a <strong>String</strong>.
	  */
	 public String getExpression() {
		  return exp;
	 }
	 /** Method to get the value of flags.
	  * @return The value of flags as a <strong>String</strong>.
	  */
	 public String getFlags() {
		  return flags;
	 }
	 /** Method to get the value of include.
	  * @param boolean The value of include as a <i>boolean</i>.
	  */
	 public boolean getInclude() {
		  return include;
	 }
	 /** Method to get the value of name.
	  * @param String The value of name as a <strong>String</string>.
	  */
	 public String getName() {
		  return name;
	 }
	 /** Method to get the name of the source of the strings used in pattern matching.
	  * @return A <strong>String</strong> which is either the fully qualified name of a metadata element, or "Filename".
	  */
	 public String getSource() {
		  if(element != null) {
				return element;
		  }
		  return "Filename";
	 }
	 /** Method to display the contents of this class as it would appear in the collection configuration file.
	  * @return A <strong>String</strong> which could be used as a subcollection entry in collect.cfg.
	  */
	 public String toString() {
		  String text = "subcollection        ";
		  text = text + name + " ";
		  text = text + "\"";
		  if(!include) {
				text = text + "!";
		  }
		  if(element != null) {
				text = text + element;
		  }
		  else {
				text = text + "Filename";
		  }
		  text = text + "/";
		  text = text + exp;
		  text = text + "/";
		  if(flags != null) {
				text = text + flags;
		  }
		  text = text + "\"\n";
		  return text;
	 }

	 /** Update certain fields of this subcollection.
	  * @param name A <strong>String</strong> which is a unique identifier of a subcollection.
	  * @param source Either the fully qualified name of the metadata whose value should be matched against the given expression, or <i>null</i> if you wish to match against the file name.
	  * @param exp A <strong>String</strong> containing a Perl expression which is used as the filter for this subcollection.
	  * @param include A <i>boolean</i> which is <i>true</i> if the condition is an include one, <i>false</i> otherwise.
	  * @param flags A <strong>String</strong> of flags to be taken into account when matching the expression.
		*/
	 public void update(String name, String source, String exp, boolean include, String flags) {
		  this.name = name;
		  this.element = source;
		  this.exp = exp;
		  this.include = include;
		  this.flags = flags;
	 }

	 /** Method to update the name of a metadata element, if it changes.
	  * @param old_name The current name of the element as a <strong>String</strong>.
	  * @param new_name The new name of the element as a <strong>String</strong>.
	  */
	 public void updateElement(String old_name, String new_name) {
		  if(element.equals(old_name)) {
				element = new_name;
		  }
	 }
}





